@@ -122,27 +122,46 @@ get.S = function(data, n, tol = 1e-5, meanzero)
   return(list(S = S, n = n))
 }
 # Algorithm 2.1: BD-MCMC algorithm for low-dimentional problem (roughly graphs with more less 8 nodes)
-bdmcmc.low = function(data, n = NULL, meanzero = FALSE, iter = 5000, burn = floor(iter / 2), skip = 1,
-gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = NULL, MCiter = 10, print = FALSE, summary = FALSE)
+bdmcmc.low = function(data, n = NULL, meanzero = FALSE, iter = 5000, burn = floor(iter / 2), skip = 1, 
+gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = "full", MCiter = 10, summary = FALSE, verbose = TRUE)
 {
+  if (is.matrix(data) == F & is.data.frame(data) == F){
+     stop("Data should be a matrix or dataframe")
+  }
+  if (is.data.frame(data) == T) data <- data.matrix(data)
+  if (any(is.na(data))) stop("Data should contain no missing data") 
   if (iter <= burn){
     stop("Number of iterations have to be more than the number of burn-in iterations")
   }
   if (gamma.b <= 0){
     stop("birth rate 'gamma.b' has to be positive value")
   }
-  id <- pmatch(prior.g, c("Uniform", "Poisson"))[1]
-  if(!is.na(id)){
-     if(id == 1) pr <- 0
-     if(id == 2) pr <- 1
-    }
   Sn <- get.S(data = data, n = n, meanzero = meanzero)
   if (is.null(Sn$n) & is.null(n)){
     stop("You have to specify the number of observations 'n'")
   }
   S <- Sn $ S
   n <- Sn $ n
-  p <- ncol(S) 
+  p <- ncol(S)
+  id <- pmatch(prior.g, c("Uniform", "Poisson"))[1]
+  if(!is.na(id)){
+     if(id == 1) pr <- 0
+     if(id == 2) pr <- 1
+    }
+  start.A <- pmatch(A, c("full", "empty", "glasso"))[1]	
+  if (!is.na(start.A)){
+     if (start.A == 1){
+        A <- 0 * S
+        A[upper.tri(A)] <- 1
+	    }
+     if (start.A == 2) A <- 0 * S
+     if (start.A == 3){
+		A <- huge(data)
+		A <- huge.select(A)
+		A <- as.matrix(Matrix(A$refit, sparse = F))
+		A[lower.tri(A)] <- 0
+		}
+    } 	
   if (is.null(A)) {
      A <- 0 * S
      A[upper.tri(A)] <- 1
@@ -161,11 +180,11 @@ gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = NULL, MCiter = 10, print
   cont <- allAcont <- 0
   alla <- ceiling(iter / 2000)# for saving allA which we need it for plotConvergency function
   for (g in 1:iter){
-    if (print == T){
-	  cat(paste("time =", format(Sys.time(), "%X")),
-      paste(c("Sum.links = ",sum(A)), collapse = ""), fill = TRUE,
-      labels = paste("{", paste(c("iter = ",g), collapse = ""),"}:",sep = ""))
-	}
+    if(verbose == T){
+	   mes <- paste(c("    MCMC iterations : ", g, " from ", iter), collapse="")
+   	   cat(mes, "\r")
+       flush.console()	
+	}     
     rates <- 0 * K
     for (i in 1:(p-1)){
      for (j in (i+1):p){
@@ -210,7 +229,13 @@ gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = NULL, MCiter = 10, print
     A[ii,jj] <- A[ii,jj] + (- 1) ^ (A[ii,jj])  
     K <- block.gibbs(K = K, A = A, bstar = bstar, Ds = Ds, gibbs.iter = 1)
   }
-    if (summary == FALSE){
+  if(verbose == TRUE){
+	mes = paste(c("    ", iter," iterations done.                              "), collapse = "")
+    cat(mes, "\r")
+    cat("\n")
+    flush.console()
+  }  
+  if (summary == FALSE){
      return(list(Ks = Ks, As = As, lambda = lambda, allA = allA, alla = alla))
   } else {
         output <- list(Ks = Ks, As = As, lambda = lambda, allA = allA, alla = alla)
@@ -249,30 +274,45 @@ get.cs_c = function(K, p, i, j){
 }
 ## Algorithm 2.1: BD-MCMC algorithm for high-dimentional problem (roughly graphs with more than 8 nodes)
 bdmcmc.high = function(data, n = NULL, meanzero = FALSE, iter = 5000, burn = floor(iter / 2),
-skip = 1, gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = NULL, print = FALSE, summary = FALSE)
+skip = 1, gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = "full", summary = FALSE, verbose = TRUE)
 {
+  if (is.matrix(data) == F & is.data.frame(data) == F){
+     stop("Data should be a matrix or dataframe")
+  }
+  if (is.data.frame(data) == T) data <- data.matrix(data)
+  if (any(is.na(data))) stop("Data should contain no missing data") 
   if (iter <= burn){
     stop("Number of iterations have to be more than the number of burn-in iterations")
   }
   if (gamma.b <= 0){
     stop("birth rate 'gamma.b' has to be positive value")
   }
-  id <- pmatch(prior.g, c("Uniform", "Poisson"))[1]
-  if(!is.na(id)){
-     if(id == 1) pr <- 0
-     if(id == 2) pr <- 1
-    }
   Sn <- get.S(data = data, n = n, meanzero = meanzero)
   if (is.null(Sn $ n) & is.null(n)){
-    stop("If you provide the covariance matrix, you have to specify the number of observations")
+    stop("If you provide the covariance matrix, you should specify the number of observations")
   }
   S <- Sn $ S
   n <- Sn $ n
-  p <- ncol(S) 
-  if (is.null(A)) {
-     A <- 0 * S
-     A[upper.tri(A)] <- 1
-     } 
+  p <- ncol(S)
+  id <- pmatch(prior.g, c("Uniform", "Poisson"))[1]
+  if (!is.na(id)){
+     if (id == 1) pr <- 0
+     if (id == 2) pr <- 1
+    }
+  start.A <- pmatch(A, c("full", "empty", "glasso"))[1]	
+  if (!is.na(start.A)){
+     if (start.A == 1){
+        A <- 0 * S
+        A[upper.tri(A)] <- 1
+	    }
+     if (start.A == 2) A <- 0 * S
+     if (start.A == 3){
+		A <- huge(data)
+		A <- huge.select(A)
+		A <- as.matrix(Matrix(A$refit, sparse = F))
+		A[lower.tri(A)] <- 0
+		}
+    } 
   if (is.null(D)) D <- diag(p)
   Ti <- chol(solve(D))
   bstar <- b + n
@@ -286,11 +326,11 @@ skip = 1, gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = NULL, print = F
   cont <- allAcont <- 0
   alla <- ceiling(iter / 2000)# for saving allA which we need it for plotConvergency function
   for (g in 1:iter){
-    if (print == T){
-	  cat(paste("time =", format(Sys.time(), "%X")),
-      paste(c("Sum.links = ", sum(A)), collapse = ""), fill = TRUE,
-      labels = paste("{", paste(c("iter = ",g), collapse = ""), "}:", sep = ""))
-	}
+    if(verbose == T){
+	   mes <- paste(c("    MCMC iterations : ", g, " from ", iter), collapse="")
+   	   cat(mes, "\r")
+       flush.console()	
+	}     
     rates <- 0 * K
     for (i in 1:(p-1)){
       for (j in (i+1):p){
@@ -331,6 +371,12 @@ skip = 1, gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = NULL, print = F
     A[ii,jj] <- A[ii,jj] + (- 1) ^ (A[ii,jj]) 
 	K <- block.gibbs(K, A, bstar, Ds, gibbs.iter = 1)
   }
+  if(verbose == TRUE){
+	mes = paste(c("    ", iter," iterations done.                              "), collapse = "")
+    cat(mes, "\r")
+    cat("\n")
+    flush.console()
+  }  
   if (summary == FALSE){
      return(list(Ks = Ks, As = As, lambda = lambda, allA = allA, alla = alla))
   } else {
@@ -357,13 +403,13 @@ skip = 1, gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = NULL, print = F
 }
 ## Main function: BDMCMC algorithm for selecting the best graphical model
 bdmcmc = function(data, n = NULL, meanzero = FALSE, iter = 5000, burn = floor(iter / 2), skip = 1,
-gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = NULL, MCiter = 10, print = FALSE, summary = FALSE)
+gamma.b = 1, prior.g = "Uniform", b = 3, D = NULL, A = "full", MCiter = 10, summary = FALSE, verbose = TRUE)
 {
   p <- ncol(data)
   if (p < 8){
-    return(bdmcmc.low(data, n, meanzero, iter, burn, skip, gamma.b, prior.g, b, D, A, MCiter, print, summary))
+    return(bdmcmc.low(data, n, meanzero, iter, burn, skip, gamma.b, prior.g, b, D, A, MCiter, summary, verbose))
   } else {
-    return(bdmcmc.high(data, n, meanzero, iter, burn, skip, gamma.b, prior.g, b, D, A, print, summary))
+    return(bdmcmc.high(data, n, meanzero, iter, burn, skip, gamma.b, prior.g, b, D, A, summary, verbose))
   }
 }
 # function for comuting probability of all links in graph
@@ -413,7 +459,7 @@ plotLinks = function(output, xlim = c(0, (nrow(output$As[[1]])) * (nrow(output$A
    ylab = "Pr(number of links in the graph|data)", xlab = "number of links in the graph")
 }
 # function for checking the convergency of the BDMCMC algorithm
-plotConvergency = function(output, skip = 1)
+plotConvergency = function(output, skip = 1, verbose = TRUE)
 {
   allA <- output$allA
   p <- nrow(allA[[1]])
@@ -426,6 +472,11 @@ plotConvergency = function(output, skip = 1)
   length.allA <- length(allA.new)
   ff <- matrix(0, p * (p - 1) / 2, length.allA)
   for (g in 1:length.allA){
+     if (verbose == TRUE){
+	    mes <- paste(c("Calculating cumulative occupancy fractions....in progress : ", floor(100 * g / length.allA), "%"), collapse = "")
+	    cat(mes, "\r")
+	    flush.console()	
+       }  
      for (k in 1:g){
         con <- 0
         for (i in 1:(p-1)){
@@ -436,6 +487,12 @@ plotConvergency = function(output, skip = 1)
         }
      }
   }
+  if(verbose == TRUE){
+	mes = paste(c("Calculating cumulative occupancy fractions....done.                   "), collapse = "")
+    cat(mes, "\r")
+    cat("\n")
+    flush.console()
+  } 
   matplot(x = skip * ((output$alla) * (1:length.allA)), y = t(ff), type = "l", lty = 1, col = 1,
   xlab = "number of iterations", ylab = "cumulative occupancy fractions for each links")
 }
@@ -452,7 +509,7 @@ plotSum = function(output, xlim = c(0, length(output$allA)))
     ylab = "sum of links in the graphs", xlab = "iterations")
 }
 # Program for computing the probability of all possible graphical models by using the result of BDMCMC algorithm
-prob.allg = function(output)
+prob.allg = function(output, verbose = TRUE)
 {
    As.lim <- output$As
    lambda <- output$lambda
@@ -462,6 +519,11 @@ prob.allg = function(output)
    i <- 0
    while (length(As.lim) > 1){
       i <- i + 1
+      if (verbose == TRUE){
+	     mes <- paste(c("    Model selection .... in progress : ", floor(100 * (1 - length(As.lim) / length(output$As))), "%"), collapse = "")
+	     cat(mes, "\r")
+	     flush.console()	
+        }  
       list.A[[i]] <- As.lim[[1]]
       list.lambda[[i]] <- vector()
       free <- 0
@@ -483,12 +545,18 @@ prob.allg = function(output)
    for (i in 1:length(list.A)){
        prob.A[i] <- sum(1 / list.lambda[[i]]) / sum(1 / lambda)
    }
+   if(verbose == TRUE){
+	 mes = paste(c("    Model selection .... done.                                  "), collapse = "")
+     cat(mes, "\r")
+     cat("\n")
+     flush.console()
+   }  
    return(list(list.A = list.A, prob.A = prob.A))
 }                                              
 # function summary of the result
-select.g = function (output, g = 2, K = FALSE)
+select.g = function (output, g = 1, K = FALSE, verbose = TRUE)
 {
-  output.allg <- prob.allg(output)
+  output.allg <- prob.allg(output, verbose = verbose)
   list.A <- output.allg$list.A
   prob.A <- output.allg$prob.A
   best.graph <- list.A[[which(prob.A == max(prob.A))]]
